﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;

using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;


namespace PruebaMapas
{
    public partial class Form1 : Form
    {
        #region private
        //Timer creation
        private Timer timer;
        //index declaration later used to get the coordenates from collection pointShapes
        private int index = -1;
        #endregion

        //collections that will store the coordenates, in the final app this will be dynamically generated for each car in the DB
        Collection<PointShape> pointShapes1;
        Collection<PointShape> pointShapes2;

        #region events
        private void Form1_Load(object sender, EventArgs e)
        {
            //Timer setting interval and handler
            timer.Interval = 300;
            timer.Tick += new EventHandler(timer_Tick);

            // Set the Map Unit. The reason for setting it to DecimalDegrees is that is what the shapefile’s unit of measure is inherently in.
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;
            
            //Restriction Layer creation
            RestrictionLayer restrictionLayer = new RestrictionLayer();
            restrictionLayer.Zones.Add(new RectangleShape(-75.33, 2.94678, -75.30, 2.92483));
            restrictionLayer.RestrictionMode = RestrictionMode.ShowZones;
            restrictionLayer.UpperScale = 250000000;
            restrictionLayer.LowerScale = double.MinValue;

            //Style for restrictionlayer
            restrictionLayer.RestrictionStyle = RestrictionStyle.UseCustomStyles;
            Style customStyle = new AreaStyle(new GeoSolidBrush(new GeoColor(150, GeoColor.StandardColors.Gray)));
            restrictionLayer.CustomStyles.Add(customStyle);

            // We create a new Layer and pass the path to a Shapefile into its constructor.
            ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(@"C:\PruebaMapas\PruebaMapas\Data\Countries02.shp");

            // Set the worldLayer with a preset Style, as AreaStyles.Country1 has YellowGreen background and black border, our worldLayer will have the same render style.
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;

            // This setting will apply from ZoonLevel01 to ZoomLevel20, that means we can see the world the same style with ZoomLevel01 all the time no matter how far we zoom out/in.
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            //Another Layer for capitals
            ShapeFileFeatureLayer capitalLayer = new ShapeFileFeatureLayer(@"C:\PruebaMapas\PruebaMapas\Data\WorldCapitals.shp");

            // Similarly, we use the presetPointStyle for cities.
            capitalLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.Capital3;

            // This setting also applies from ZoomLevel01 to ZoomLevel20, that means we can see city symbols the same style with ZoomLevel01 all the time.
            capitalLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
                        
            //ImMemory Feature Layer wich contains the shape that will be used as container for the tif map
            InMemoryFeatureLayer inMemoryFeatureLayer = new InMemoryFeatureLayer();
            inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.Transparent, GeoColor.StandardColors.Transparent, 4);
            inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            //Shape used as container
           // RectangleShape containerNeiva = new RectangleShape(-75.34450, 2.99291, -75.21007, 2.88143);
            RectangleShape containerComuna3 = new RectangleShape(-75.33, 2.94678, -75.30, 2.92483);

            //inMemoryFeatureLayer.InternalFeatures.Add(new Feature(containerNeiva));
            inMemoryFeatureLayer.InternalFeatures.Add(new Feature(containerComuna3));

            //Layer for the tif file
            //GdiPlusRasterLayer tifNeiva = new GdiPlusRasterLayer(@"C:\PruebaMapas\PruebaMapas\Data\ScreenshotMapaNeiva.tif", containerNeiva);
            GdiPlusRasterLayer tifComuna3 = new GdiPlusRasterLayer(@"C:\PruebaMapas\PruebaMapas\Data\NEIVA_COMUNA_3_160210.tif",containerComuna3);

            // Create a new Layer Overlay to hold the layers we just created
            LayerOverlay layerOverlay = new LayerOverlay();

            // Create a new Layer Overlay to hold the car image layer
            LayerOverlay carOverlay = new LayerOverlay();

            //create a layer for the restriction layer
            LayerOverlay restrictionOverlay = new LayerOverlay();
           
            //Layer for the cars' image
            InMemoryFeatureLayer bitmapLayer = new InMemoryFeatureLayer();
            bitmapLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.PointType = PointType.Bitmap;
            bitmapLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.Image = new GeoImage(@"C:\PruebaMapas\PruebaMapas\Data\vehicle1.PNG");
            bitmapLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.ContestedBorder2;
            bitmapLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            bitmapLayer.InternalFeatures.Add("Car1",new Feature(new PointShape(-75.3099,2.93)));           
            bitmapLayer.InternalFeatures.Add("Car2", new Feature(new PointShape(-75.3156, 2.932)));
            
            // Add the shapefile and tiff layers to the layeroverlay
            layerOverlay.Layers.Add(worldLayer);
            layerOverlay.Layers.Add(capitalLayer);
            layerOverlay.Layers.Add("ImageLayerBoundingBox", inMemoryFeatureLayer);
            //layerOverlay.Layers.Add("GdiPlusImageLayer", tifNeiva);
            layerOverlay.Layers.Add("GdiPlusImageLayer2", tifComuna3);

            //Add the bitmap layer to the layer caroverlay
            carOverlay.Layers.Add("BitmapLayer", bitmapLayer);

            //Add the restriction layer to the restrictionoverlay
            restrictionOverlay.Layers.Add("RestrictionLayer", restrictionLayer);

            // We need to add the layers layeroverlay, caroverlay and restrictionoverlay to Map.
            winformsMap1.Overlays.Add(layerOverlay);
            winformsMap1.Overlays.Add("CarOverlay", carOverlay);
            winformsMap1.Overlays.Add("RestrictionOverlay", restrictionOverlay);

            // Set a proper extent for the Map.
            winformsMap1.CurrentExtent = new RectangleShape(-75.32, 2.9362, -75.31, 2.931);
            //background style
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean);
        
       
            // We now need to call the Refresh() method of the Map control so that the Map can redraw based on the data that has been provided.
            winformsMap1.Refresh();
            timer.Start();
        }

        void timer_Tick(object sender, EventArgs e)
        {
            InMemoryFeatureLayer bitmapLayer = (InMemoryFeatureLayer)winformsMap1.FindFeatureLayer("BitmapLayer");
            //this pointShapes will be dynamically generated in the final app for each car in the DB
            PointShape pointShape1 = bitmapLayer.InternalFeatures[0].GetShape() as PointShape;
            PointShape pointShape2 = bitmapLayer.InternalFeatures[1].GetShape() as PointShape;
            //LineShape airLine = ((MultilineShape)bitmapLayer.InternalFeatures[1].GetShape()).Lines[0];
            pointShapes1 = ReadTextFile(@"C:\PruebaMapas\PruebaMapas\Data\GPSreadings1.txt");
            pointShapes2 = ReadTextFile(@"C:\PruebaMapas\PruebaMapas\Data\GPSreadings2.txt");
            index += 1;
            if (index < pointShapes1.Count)
            {
               //redraws the car in the next position given by the coordenates in pointShapes collection
                // Vertex newPosition = pointShapes[index];
                pointShape1.X = pointShapes1[index].X;
                pointShape1.Y = pointShapes1[index].Y;
                pointShape1.Id = "Car1";
                pointShape2.X = pointShapes2[index].X;
                pointShape2.Y = pointShapes2[index].Y;
                pointShape2.Id = "Car2";


                bitmapLayer.Open();
                bitmapLayer.EditTools.BeginTransaction();
                bitmapLayer.EditTools.Update(pointShape1);
                bitmapLayer.EditTools.Update(pointShape2);
                bitmapLayer.EditTools.CommitTransaction();
                bitmapLayer.Close();
            }
            else
            {
                index = 0;
            }
            
            winformsMap1.Refresh(winformsMap1.Overlays["CarOverlay"]);
        }

        private Collection<PointShape> ReadTextFile(string textFile)
        {
            Collection<PointShape> pointShapes = new Collection<PointShape>();
            StreamReader SR = File.OpenText(textFile);
            String[] coordenates;
            string S;
            S = SR.ReadLine();
            while (S != null)
            {
                coordenates = S.Split(",".ToCharArray());
                double X = System.Convert.ToDouble(coordenates[0].Replace(".",","));
                double Y = System.Convert.ToDouble(coordenates[1].Replace(".",","));
                //double Angle = System.Convert.ToDouble(coordenates[2].Replace(".",","));
                PointShape pointShape = new PointShape(X, Y);
                pointShapes.Add(pointShape);
                S = SR.ReadLine();
            }
            SR.Close();
            return pointShapes;
        }

        private void winformsMap1_MouseMove(object sender, MouseEventArgs e)
        {
            //Displays the X and Y in screen coordinates.
            statusStrip1.Items["toolStripStatusLabelScreen"].Text = "X:" + e.X + " Y:" + e.Y;

            //Gets the PointShape in world coordinates from screen coordinates.
            PointShape pointShape = ExtentHelper.ToWorldCoordinate(winformsMap1.CurrentExtent, new ScreenPointF(e.X, e.Y), winformsMap1.Width, winformsMap1.Height);

            //Displays world coordinates.
            statusStrip1.Items["toolStripStatusLabelWorld"].Text = "(world) X:" + Math.Round(pointShape.X, 4) + " Y:" + Math.Round(pointShape.Y, 4);
        }
        #endregion
        public Form1()
        {
            InitializeComponent();
            timer = new Timer();
        }
    }
}
